@@ -1,19 +1,18 @@
 /* debug.h: Prototypes and declarations for debug.c
- * Copyright (C) 2006 Julian Graham
+ * Copyright (C) 2011 Julian Graham
  *
- * libRUIN is free software; you can redistribute it and/or modify
+ * libRUIN is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * libRUIN is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with libRUIN; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef RUIN_DEBUG_H
@@ -22,7 +21,8 @@
 #include <libguile.h>
 
 #include "layout.h"
+#include "parse.h"
 
-SCM ruin_debug_dump_layout_tree(ruin_element_t *);
+void ruin_debug_print_box_tree (ruin_box_t *);
 
-#endif
+#endif /* RUIN_DEBUG_H */
