@@ -1,21 +1,23 @@
 /* util.c: Utility functions for libRUIN
- * Copyright (C) 2007 Julian Graham
+ * Copyright (C) 2011 Julian Graham
  *
- * libRUIN is free software; you can redistribute it and/or modify
+ * libRUIN is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * libRUIN is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with libRUIN; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define _XOPEN_SOURCE
+#define _GNU_SOURCE
+
 #include <ctype.h>
 #include <libguile.h>
 #include <math.h>
@@ -31,6 +33,14 @@
 #include "util.h"
 #include "window.h"
 
+static char *roman_strings[3][9] = {
+  { "I", "II", "III", "IV", "V", "VI", "VII", "LVII", "IX" },
+  { "X", "XX", "XXX", "XL", "L", "LX", "LXX", "LXXX", "XC" },
+  { "C", "CC", "CCC", "CD", "D", "DC", "DCC", "DCCC", "CM" },
+};
+
+static int roman_string_lengths[] = { 1, 2, 3, 2, 1, 2, 3, 4, 2 };
+
 pthread_mutex_t _ruin_util_id_lock;
 
 int ruin_util_hash_hash (char *k, int table_size) {
@@ -100,7 +110,6 @@ void ruin_util_hash_insert(ruin_util_hash *hash, char *key, void *value) {
   else {
     mapped_index = hash->map[hashed_index];
     if (strcmp(hash->keys[mapped_index], key) == 0) {
-      free(hash->values[mapped_index]);
       hash->values[mapped_index] = value;
       pthread_mutex_unlock(hash->lock);
       return;
@@ -112,7 +121,6 @@ void ruin_util_hash_insert(ruin_util_hash *hash, char *key, void *value) {
 	return;
       }
       else if(strcmp(hash->keys[hash->map[i]], key) == 0) {
-	free(hash->values[hash->map[i]]);
 	hash->values[hash->map[i]] = value;
 	pthread_mutex_unlock(hash->lock);
 	return;
@@ -125,7 +133,6 @@ void ruin_util_hash_insert(ruin_util_hash *hash, char *key, void *value) {
 	return;
       }
       else if(strcmp(hash->keys[hash->map[i]], key) == 0) {
-	free(hash->values[hash->map[i]]);
 	hash->values[hash->map[i]] = value;
 	pthread_mutex_unlock(hash->lock);
 	return;
@@ -311,49 +318,6 @@ void ruin_util_hash_free(ruin_util_hash *hash) {
   return;
 }
 
-/* Here are the list functions. */
-
-ruin_util_list *ruin_util_list_new(void *data) {
-  ruin_util_list *foo = calloc(1, sizeof(ruin_util_list));
-  foo->data = data;
-  return foo;
-}
- 
-int ruin_util_list_length(ruin_util_list *list) {
-  if (list != NULL)
-    return 1 + ruin_util_list_length(list->next);
-  return 0;
-}
-
-ruin_util_list *ruin_util_list_push_front(ruin_util_list *head,
-					  ruin_util_list *new_head) {
-  new_head->next = head;
-  return new_head;
-}
-
-ruin_util_list *ruin_util_list_append(ruin_util_list *head,
-				      ruin_util_list *new_tail) {
-  if (head == NULL)
-    return new_tail;
-  if (head->next != NULL)
-    (void) ruin_util_list_append(head->next, new_tail);
-  else head->next = new_tail;
-  return head;
-}
-
-ruin_util_list *ruin_util_list_get_ith(ruin_util_list *head, int i) {
-  if (head == NULL || i == 0)
-    return head;
-  return ruin_util_list_get_ith(head->next, i - 1);
-}
-
-void ruin_util_list_free(ruin_util_list *head) {
-  ruin_util_list *next = head->next;
-  free(head);
-  if (next != NULL)
-    ruin_util_list_free(next);
-}
-
 /* And some string conversion functions. */
 
 char *ruin_util_int_to_string (int fd) {
@@ -418,52 +382,114 @@ long ruin_util_generate_id() {
 }
 
 void ruin_util_log(ruin_window_t *w, char *msg, ...) {
+  FILE *output = (w == NULL) ? stderr : w->log;
+  char str[64];
+
+  struct timeval tv = { 0, 0 };
+  struct tm time;
   va_list args;
+
+  (void) gettimeofday(&tv, NULL);
+  snprintf(str, 64, "%d", (int) tv.tv_sec);
+  (void) strptime(str, "%s", &time);
+  strftime(str, 64, "%a, %d %b %Y %H:%M:%S", &time);
+  fprintf(output, "%s.%.3ld; ", str, tv.tv_usec / 10000);
+
   va_start(args, msg);
-  if (w->log != NULL) {
-    fprintf(w->log, "window %ld: ", w->internal_id);
-    vfprintf(w->log, msg, args);
-    fprintf(w->log, "\n");
-    fflush(w->log); 
+  
+  if (output != NULL) {
+    if (w != NULL)
+      fprintf(output, "window %ld: ", w->internal_id);
+    else fprintf(output, "libruin-root: ");
+    vfprintf(output, msg, args);
+    fprintf(output, "\n");
+    fflush(output); 
   }
+
   va_end(args);
 }
 
-/* Somewhat inefficient; refactor to minimize use of realloc()? */
+char *ruin_util_arabic_to_roman (int n, int use_upper) 
+{
+  char *str = NULL;
+  int i = 0, len = ruin_util_arabic_to_roman_length (n);
 
-char *ruin_util_arabic_to_roman(int arabic, int use_upper) {
-  int i = 0, len = 0;
-  char *ret = NULL;
-  static int digs[] = { 1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1 };
-  static char *strsu[] = 
-    { "M", "CM", "D", "CD", "C", "XC", "L", "XL", "X", "IX", "V", "IV", "I" };
-  static char *strsl[] =
-    { "m", "cm", "d", "cd", "c", "xc", "l", "xl", "x", "ix", "v", "iv", "i" };
-  if (arabic > 1000)
+  if (n > 4096 || n < 0)
     return NULL;
-  for (i = 0; i < 13; i++) {
-    while (arabic >= digs[i]) {
-      int slen = strlen(strsu[i]);
-      arabic -= digs[i];
-      ret = realloc(ret, len + slen + 1);
-      strncpy(ret + len, use_upper ? strsu[i] : strsl[i], slen);
-      len += slen;
+
+  str = calloc (len + 1, sizeof (char));
+
+  if (n >= 1000)
+    {
+      i = n / 1000;
+      memset (str, 'M', i);
+      n -= i * 1000;
     }
-  }
-  ret = realloc(ret, len + 1);
-  ret[len] = 0;
-  return ret;
+  if (n >= 100)
+    {
+      i = n / 100;
+      strncat (str, roman_strings[2][i - 1], roman_string_lengths[i - 1]);
+      n -= i * 100;
+    }
+  if (n >= 10)
+    {
+      i = n / 10;
+      strncat (str, roman_strings[1][i - 1], roman_string_lengths[i - 1]);
+      n -= i * 10;
+    }
+  if (n >= 1)
+    strncat (str, roman_strings[0][n - 1], roman_string_lengths[n - 1]);
+
+  if (!use_upper)
+    for (i = 0; i < len; i++)
+      str[i] = tolower (str[i]);
+
+  return str;
 }
 
-char *ruin_util_get_parent_directory(char *filename) {
+int ruin_util_arabic_to_roman_length (int n)
+{
+  int i = 0;
+  int num_digits = 0;
+
+  if (n > 4096 || n < 0)
+    return -1;
+
+  if (n >= 1000)
+    {
+      i = n / 1000;
+      num_digits += i;
+      n -= i * 1000;
+    }
+  if (n >= 100)
+    {
+      i = n / 100;
+      num_digits += roman_string_lengths[i - 1];
+      n -= i * 100;
+    }
+  if (n >= 10)
+    {
+      i = n / 10;
+      num_digits += roman_string_lengths[i - 1];
+      n -= i * 10;
+    }
+  if (n >= 1)
+    num_digits += roman_string_lengths[n - 1];
+  
+  return num_digits;
+}
+
+char *ruin_util_get_parent_directory (char *filename) 
+{
   char *rstr = NULL;
-  char *abs_path = malloc(sizeof(char) * PATH_MAX);
+  char *abs_path = malloc (sizeof (char) * PATH_MAX);
   char *ret = NULL;
-  realpath(filename, abs_path);
-  rstr = strrchr(abs_path, '/');
-  ret = calloc(rstr - abs_path + 1, sizeof(char));
-  strncat(ret, abs_path, rstr - abs_path);
-  free(abs_path);
+
+  abs_path = realpath (filename, abs_path);
+  rstr = strrchr (abs_path, '/');
+  ret = calloc (rstr - abs_path + 1, sizeof (char));
+  strncat (ret, abs_path, rstr - abs_path);
+  free (abs_path);
   return ret;
 }
 
@@ -473,6 +499,29 @@ long ruin_util_current_time_millis() {
   return (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
 }
 
-SCM ruin_util_string2scm(char *str) {
-  return scm_mem2string(str, strlen(str));
+GList *ruin_util_map (GList *l, gpointer (*func)(gpointer))
+{
+  GList *ret = NULL;
+  GList *l_ptr = l;
+  while (l_ptr != NULL)
+    {
+      ret = g_list_append (ret, func (l_ptr->data));
+      l_ptr = l_ptr->next;
+    }
+
+  return ret;
+}
+
+GList *ruin_util_map_context 
+(GList *l, gpointer (*func)(gpointer, gpointer), gpointer user_data)
+{
+  GList *ret = NULL;
+  GList *l_ptr = l;
+  while (l_ptr != NULL)
+    {
+      ret = g_list_append (ret, func (l_ptr->data, user_data));
+      l_ptr = l_ptr->next;
+    }
+
+  return ret;
 }
